﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Interactivity;
using System.Windows.Media.Imaging;
using System.Windows.Data;
using System.Diagnostics;
using System.Linq;

namespace ASPItalia.Books.Chapter07.Behaviors
{
    public class ImageWebRequestBehavior : Behavior<Image>
    {

        public static readonly DependencyProperty SourceUriProperty = DependencyProperty.Register("SourceUri", typeof(Binding), typeof(ImageWebRequestBehavior), new PropertyMetadata(null, OnSourceUriChanged));

        private static readonly DependencyProperty BindingListenerProperty = DependencyProperty.RegisterAttached("BindingListener", typeof(Uri), typeof(ImageWebRequestBehavior), new PropertyMetadata(OnBindingListenerChanged));

        private WebClient client;
        private BitmapImage bitmapImage;

        public ImageWebRequestBehavior()
        {
            this.bitmapImage = new BitmapImage();
        }

        private static void OnBindingListenerChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            Image image = (Image)d;

            // Cerco il behavior applicato all'immagine
            ImageWebRequestBehavior behavior = Interaction.GetBehaviors(image).OfType<ImageWebRequestBehavior>().First();
            // Applico il nuovo URI
            behavior.LoadUri((Uri)e.NewValue);
        }

        public Binding SourceUri
        {
            get { return (Binding)this.GetValue(SourceUriProperty); }
            set { this.SetValue(SourceUriProperty, value); }
        }

        protected override void OnAttached()
        {
            base.OnAttached();

            SetBinding();
        }

        private static void OnSourceUriChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ImageWebRequestBehavior behavior = (ImageWebRequestBehavior)d;

            behavior.SetBinding();
        }

        private void SetBinding()
        {
            if (this.AssociatedObject != null)
                this.AssociatedObject.SetBinding(BindingListenerProperty, this.SourceUri);
        }

        private void LoadUri(Uri uri)
        {
            // Annullo la richiesta web non conclusa
            if (client != null)
                client.CancelAsync();

            // Preparo una richiesta web per l'indirizzo
            client = new WebClient();
            client.AllowReadStreamBuffering = true;
            client.OpenReadCompleted += (s, e) => client_OpenReadCompleted(uri, e);
            client.OpenReadAsync(uri);
        }

        private void client_OpenReadCompleted(Uri uri, OpenReadCompletedEventArgs e)
        {
            // Se si è verificato qualche errore o l'operazione
            // è conclusa, non faccio niente
            if (e.Cancelled || e.Error != null) return;

            try
            {
                // Carico l'immagine
                using (e.Result)
                    this.bitmapImage.SetSource(e.Result);
                this.AssociatedObject.Source = bitmapImage;
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Error for image uri {0}: {1}", uri, ex.Message);
            }
        }

    }

}
