﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ASPItalia.Books.Chapter07.Components;
using System.Diagnostics;
using System.Windows.Media.Imaging;

namespace ASPItalia.Books.Chapter07
{
    public partial class VirtualItem : UserControl
    {

        public static readonly DependencyProperty ItemProperty = DependencyProperty.Register("Item", typeof(Product), typeof(VirtualItem), new PropertyMetadata(OnItemChanged));

        // Per debug: conta le istanze
        private static int instanceCount;

        public VirtualItem()
        {
            InitializeComponent();

            Debug.WriteLine("Istanza UI ", ++instanceCount);
        }

        public Product Item
        {
            get { return this.GetValue(ItemProperty) as Product; }
            set { this.SetValue(ItemProperty, value); }
        }

        private static void OnItemChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((VirtualItem)d).ItemChanged();
        }

        private void ItemChanged()
        {
            Product p = this.Item;

            if (p == Product.LoadItem)
            {
                // Mostro il loading e nascondo i testi
                this.loading.Visibility = Visibility.Visible;
                this.quantityBorder.Visibility = Visibility.Collapsed;
                this.description.Visibility = Visibility.Collapsed;
                this.image.Visibility = Visibility.Collapsed;
            }
            else
            {
                // Nascondo il loading e mostro i testi
                this.loading.Visibility = Visibility.Collapsed;
                this.quantityBorder.Visibility = Visibility.Visible;
                this.description.Visibility = Visibility.Visible;
                this.image.Visibility = Visibility.Visible;

                // Decido il colore da utilizzare in base alla quantità
                Color color = Colors.Green;
                if (p.Quantity < 10)
                    color = Colors.Blue;
                else if (p.Quantity < 30)
                    color = Colors.DarkGray;

                this.quantityBorder.Background = new SolidColorBrush(color);
                // Imposto i testi
                this.quantity.Text = p.Quantity.ToString();
                this.description.Text = p.Description;

                this.LoadImage(p.Image);
            }
        }

        private WebClient client;

        private void LoadImage(Uri uri)
        {
            // Annullo la richiesta web non conclusa
            if (client != null)
                client.CancelAsync();

            // Preparo una richiesta web per l'indirizzo
            client = new WebClient();
            client.AllowReadStreamBuffering = true;
            client.OpenReadCompleted += (s, e) => client_OpenReadCompleted(uri, e);
            client.OpenReadAsync(uri);
        }

        private void client_OpenReadCompleted(Uri uri, OpenReadCompletedEventArgs e)
        {
            // Se si è verificato qualche errore o l'operazione
            // è conclusa, non faccio niente
            if (e.Cancelled || e.Error != null) return;

            try
            {
                // Carico l'immagine
                using (e.Result)
                    this.bitmapImage.SetSource(e.Result);
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Error for image uri {0}: {1}", uri, ex.Message);
            }
        }

    }
}
