﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using ASPItalia.Books.Chapter07.Components;
using System.Threading;
using System.Diagnostics;

namespace ASPItalia.Books.Chapter07
{
    public partial class VirtualPage : PhoneApplicationPage
    {

        private VirtualList<Product> list;
        private Uri[] imagesUri;

        public VirtualPage()
        {
            InitializeComponent();

            // Preparo la lista virtuale
            this.list = new VirtualList<Product>(Product.LoadItem);
            this.list.ItemsNeeded += list_ItemsNeeded;

            // Associo la lista alla ListBox
            this.listBox.ItemsSource = list;

            this.imagesUri = new Uri[] { 
                new Uri("http://books.aspitalia.com/Windows-Phone/cover.jpg"),
                new Uri("http://books.aspitalia.com/ASP.NET-4.0/cover.jpg"),
                new Uri("http://books.aspitalia.com/VisualBasic-2010/cover.jpg"),
                new Uri("http://books.aspitalia.com/ASP.NET-4.0-in-practice/cover.jpg"),
                new Uri("http://books.aspitalia.com/CSharp-4/cover.jpg"),
                new Uri("http://books.aspitalia.com/Entity-Framework/cover.jpg"),
                new Uri("http://books.aspitalia.com/CSharp-4-Guida-Professionale/cover.jpg"),
                new Uri("http://books.aspitalia.com/ASP.NET-4-Guida-Professionale/cover.jpg"),
                new Uri("http://books.aspitalia.com/Silverlight-3.0/cover.jpg"),
                new Uri("http://books.aspitalia.com/ASP.NET-3.5/cover.jpg"),
                new Uri("http://books.aspitalia.com/ASP.NET-2.0/cover.jpg"),
                new Uri("http://books.aspitalia.com/ASP-3-per-esempi/cover.jpg"),
            };
        }

        private void list_ItemsNeeded(object sender, ItemsNeededEventArgs e)
        {
            Debug.WriteLine("Asked items from {0}", e.StartIndex);

            // Carico in asincrono, per non pesare sulla UI
            ThreadPool.QueueUserWorkItem(c => LoadNextChunk(e.StartIndex));
        }

        private void LoadNextChunk(int index)
        {
            // Finta pausa di caricamento
            System.Threading.Thread.Sleep(2000);

            Random random = new Random(Environment.TickCount);
            // Preparo dei dati fittizzi da caricare
            var newItems = Enumerable.Range(index + 1, 30)
                .Select(r => new Product
                {
                    Description = "Prodotto " + r,
                    Quantity = random.Next(0, 50),
                    Image = imagesUri[random.Next(0, imagesUri.Length - 1)]
                }).ToArray();

            // Aggiungo gli elementi (sul thread UI, importante)
            this.Dispatcher.BeginInvoke(() => this.list.AddRange(newItems));
        }

    }
}