﻿Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Ink
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports System.Windows.Interactivity
Imports System.Windows.Media.Imaging
Imports System.Windows.Data
Imports System.Diagnostics
Imports System.Linq

Namespace Behaviors
    Public Class ImageWebRequestBehavior
        Inherits Behavior(Of Image)

        Public Shared ReadOnly SourceUriProperty As DependencyProperty = DependencyProperty.Register("SourceUri", GetType(Binding), GetType(ImageWebRequestBehavior), New PropertyMetadata(Nothing, AddressOf OnSourceUriChanged))

        Private Shared ReadOnly BindingListenerProperty As DependencyProperty = DependencyProperty.RegisterAttached("BindingListener", GetType(Uri), GetType(ImageWebRequestBehavior), New PropertyMetadata(AddressOf OnBindingListenerChanged))

        Private client As WebClient
        Private bitmapImage As BitmapImage

        Public Sub New()
            Me.bitmapImage = New BitmapImage()
        End Sub

        Private Shared Sub OnBindingListenerChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
            Dim image As Image = DirectCast(d, Image)

            ' Cerco il behavior applicato all'immagine
            Dim behavior As ImageWebRequestBehavior = Interaction.GetBehaviors(image).OfType(Of ImageWebRequestBehavior)().First()
            ' Applico il nuovo URI
            behavior.LoadUri(DirectCast(e.NewValue, Uri))
        End Sub

        Public Property SourceUri() As Binding
            Get
                Return DirectCast(Me.GetValue(SourceUriProperty), Binding)
            End Get
            Set(ByVal value As Binding)
                Me.SetValue(SourceUriProperty, value)
            End Set
        End Property

        Protected Overrides Sub OnAttached()
            MyBase.OnAttached()

            SetBinding()
        End Sub

        Private Shared Sub OnSourceUriChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
            Dim behavior As ImageWebRequestBehavior = DirectCast(d, ImageWebRequestBehavior)

            behavior.SetBinding()
        End Sub

        Private Sub SetBinding()
            If Me.AssociatedObject IsNot Nothing Then
                Me.AssociatedObject.SetBinding(BindingListenerProperty, Me.SourceUri)
            End If
        End Sub

        Private Sub LoadUri(ByVal uri As Uri)
            ' Annullo la richiesta web non conclusa
            If client IsNot Nothing Then
                client.CancelAsync()
            End If

            ' Preparo una richiesta web per l'indirizzo
            client = New WebClient()
            client.AllowReadStreamBuffering = True
            AddHandler client.OpenReadCompleted, Sub(s, e) client_OpenReadCompleted(uri, e)
            client.OpenReadAsync(uri)
        End Sub

        Private Sub client_OpenReadCompleted(ByVal uri As Uri, ByVal e As OpenReadCompletedEventArgs)
            ' Se si è verificato qualche errore o l'operazione
            ' è conclusa, non faccio niente
            If e.Cancelled OrElse e.[Error] IsNot Nothing Then
                Return
            End If

            Try
                ' Carico l'immagine
                Using e.Result
                    Me.bitmapImage.SetSource(e.Result)
                End Using
                Me.AssociatedObject.Source = bitmapImage
            Catch ex As Exception
                Debug.WriteLine("Error for image uri {0}: {1}", uri, ex.Message)
            End Try
        End Sub

    End Class

End Namespace