﻿Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Ink
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports System.Collections
Imports System.Collections.Generic
Imports System.Collections.Specialized
Imports System.Diagnostics

Namespace Components

 
    Public Class VirtualList(Of T)
        Implements IList(Of T)
        Implements ICollection(Of T)
        Implements INotifyCollectionChanged

        Private syncObject As New Object()
        Private loadedList As New List(Of T)()
        Private askedStartIndex As New List(Of Int32)()
        Private loadItem As T

        Public Event CollectionChanged As NotifyCollectionChangedEventHandler Implements INotifyCollectionChanged.CollectionChanged

        Public Event ItemsNeeded As EventHandler(Of ItemsNeededEventArgs)

        Public Sub New(ByVal loadItem As T)
            Me.loadItem = loadItem
        End Sub

#Region "Not supported"

        Public Sub Clear() Implements ICollection(Of T).Clear
            Throw New NotSupportedException()
        End Sub

        Public Sub Insert(ByVal index As Integer, ByVal value As T) Implements IList(Of T).Insert
            Throw New NotSupportedException()
        End Sub

        Public Sub CopyTo(ByVal array As T(), ByVal index As Integer) Implements ICollection(Of T).CopyTo
            Throw New NotSupportedException()
        End Sub

        Public Function Remove(ByVal value As T) As Boolean Implements ICollection(Of T).Remove
            Throw New NotSupportedException()
        End Function

        Public Sub RemoveAt(ByVal index As Integer) Implements IList(Of T).RemoveAt
            Throw New NotSupportedException()
        End Sub

#End Region

#Region "Fixed implementation"

        Public ReadOnly Property IsFixedSize() As Boolean
            Get
                Return True
            End Get
        End Property

        Public ReadOnly Property IsReadOnly() As Boolean Implements ICollection(Of T).IsReadOnly
            Get
                Return True
            End Get
        End Property

        Public ReadOnly Property IsSynchronized() As Boolean
            Get
                Return False
            End Get
        End Property

        Private Function IEnumerable_GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
            Return Me.GetEnumerator()
        End Function

#End Region

#Region "Protected"

        Protected Overridable Sub OnItemsNeeded(ByVal e As ItemsNeededEventArgs)
            RaiseEvent ItemsNeeded(Me, e)
        End Sub

        Protected Overridable Sub OnCollectionChanged(ByVal e As NotifyCollectionChangedEventArgs)
            RaiseEvent CollectionChanged(Me, e)
        End Sub

#End Region

        Private Sub AskItems()
            Dim index As Integer = Me.loadedList.Count

            ' Controllo che non abbia già chiesto di caricare da questo indice
            If Not Me.askedStartIndex.Contains(index) Then
                Me.askedStartIndex.Add(index)

                ' Metto un item che indica il caricamento
                Me.Add(loadItem)

                Me.OnItemsNeeded(New ItemsNeededEventArgs(index))
            End If
        End Sub

        Public Sub Add(ByVal value As T) Implements ICollection(Of T).Add
            Me.loadedList.Add(value)

            Dim args As New NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Add, value, Me.loadedList.Count - 1)
            Me.OnCollectionChanged(args)
        End Sub

        Public Sub AddRange(ByVal items As IEnumerable(Of T))
            ' Rimuovo l'elemento di caricamento, se presente
            RemoveLoadItem()

            For Each item As T In items
                Me.Add(item)
            Next
        End Sub

        Private Sub RemoveLoadItem()
            Dim lastIndex As Integer = Me.loadedList.Count - 1
            If lastIndex >= 0 AndAlso Me.loadedList(lastIndex).Equals(Me.loadItem) Then
                Dim item As T = Me.loadedList(lastIndex)
                Me.loadedList.RemoveAt(lastIndex)

                Dim args As New NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Remove, item, lastIndex)
                Me.OnCollectionChanged(args)
            End If
        End Sub

        Public Function Contains(ByVal value As T) As Boolean Implements ICollection(Of T).Contains
            Return Me.loadedList.Contains(value)
        End Function

        Public Function IndexOf(ByVal value As T) As Integer Implements IList(Of T).IndexOf
            Return Me.loadedList.IndexOf(value)
        End Function

        Default Public Property Item(ByVal index As Integer) As T Implements IList(Of T).Item
            Get
                ' Se sono all'ultimo alemento e non è l'elemento di caricamento, ne chiedo degli altri
                If index = Me.loadedList.Count - 1 AndAlso Not Me.loadedList(index).Equals(Me.loadItem) Then
                    Me.AskItems()
                End If
                Return Me.loadedList(index)
            End Get
            Set(ByVal value As T)
                Throw New NotSupportedException()
            End Set
        End Property

        Public ReadOnly Property Count() As Integer Implements ICollection(Of T).Count
            Get
                ' Mi assicuro che abbia chiesto almeno il caricamento
                ' iniziale della lista
                If Not askedStartIndex.Contains(0) Then
                    AskItems()
                End If
                Return loadedList.Count
            End Get
        End Property

        Public ReadOnly Property SyncRoot() As Object
            Get
                Return syncObject
            End Get
        End Property

        Public Function GetEnumerator() As IEnumerator(Of T) Implements IEnumerable(Of T).GetEnumerator
            Throw New NotImplementedException()
        End Function

    End Class

    Public Class ItemsNeededEventArgs
        Inherits EventArgs

        Public Sub New(ByVal startIndex As Integer)
            Me.StartIndex = startIndex
        End Sub

        Public Property StartIndex() As Integer
            Get
                Return m_StartIndex
            End Get
            Private Set(ByVal value As Integer)
                m_StartIndex = value
            End Set
        End Property
        Private m_StartIndex As Integer
    End Class
End Namespace