﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports ASPItalia.Books.Chapter07.Components
Imports System.Diagnostics
Imports System.Windows.Media.Imaging

Partial Public Class VirtualItem
    Inherits UserControl

    Public Shared ReadOnly ItemProperty As DependencyProperty = DependencyProperty.Register("Item", GetType(Product), GetType(VirtualItem), New PropertyMetadata(AddressOf OnItemChanged))

    ' Per debug: conta le istanze
    Private Shared instanceCount As Integer

    Public Sub New()
        InitializeComponent()

        Debug.WriteLine("Istanza UI ", System.Threading.Interlocked.Increment(instanceCount))
    End Sub

    Public Property Item() As Product
        Get
            Return TryCast(Me.GetValue(ItemProperty), Product)
        End Get
        Set(ByVal value As Product)
            Me.SetValue(ItemProperty, value)
        End Set
    End Property

    Private Shared Sub OnItemChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
        DirectCast(d, VirtualItem).ItemChanged()
    End Sub

    Private Sub ItemChanged()
        Dim p As Product = Me.Item

        If p Is Product.LoadItem Then
            ' Mostro il loading e nascondo i testi
            Me.loading.Visibility = Visibility.Visible
            Me.quantityBorder.Visibility = Visibility.Collapsed
            Me.description.Visibility = Visibility.Collapsed
            Me.image.Visibility = Visibility.Collapsed
        Else
            ' Nascondo il loading e mostro i testi
            Me.loading.Visibility = Visibility.Collapsed
            Me.quantityBorder.Visibility = Visibility.Visible
            Me.description.Visibility = Visibility.Visible
            Me.image.Visibility = Visibility.Visible

            ' Decido il colore da utilizzare in base alla quantità
            Dim color As Color = Colors.Green
            If p.Quantity < 10 Then
                color = Colors.Blue
            ElseIf p.Quantity < 30 Then
                color = Colors.DarkGray
            End If

            Me.quantityBorder.Background = New SolidColorBrush(color)
            ' Imposto i testi
            Me.quantity.Text = p.Quantity.ToString()
            Me.description.Text = p.Description

            Me.LoadImage(p.Image)
        End If
    End Sub

    Private client As WebClient

    Private Sub LoadImage(ByVal uri As Uri)
        ' Annullo la richiesta web non conclusa
        If client IsNot Nothing Then
            client.CancelAsync()
        End If

        ' Preparo una richiesta web per l'indirizzo
        client = New WebClient()
        client.AllowReadStreamBuffering = True
        AddHandler client.OpenReadCompleted, Sub(s, e) client_OpenReadCompleted(uri, e)
        client.OpenReadAsync(uri)
    End Sub

    Private Sub client_OpenReadCompleted(ByVal uri As Uri, ByVal e As OpenReadCompletedEventArgs)
        ' Se si è verificato qualche errore o l'operazione
        ' è conclusa, non faccio niente
        If e.Cancelled OrElse e.[Error] IsNot Nothing Then
            Return
        End If

        Try
            ' Carico l'immagine
            Using e.Result
                Me.bitmapImage.SetSource(e.Result)
            End Using
        Catch ex As Exception
            Debug.WriteLine("Error for image uri {0}: {1}", uri, ex.Message)
        End Try
    End Sub

End Class