﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using System.Diagnostics;
using System.IO.IsolatedStorage;
using System.Xml.Serialization;

namespace ASPItalia.Books.Chapter8
{
  public partial class App : Application
  {
    /// <summary>
    /// Provides easy access to the root frame of the Phone Application.
    /// </summary>
    /// <returns>The root frame of the Phone Application.</returns>
    public PhoneApplicationFrame RootFrame { get; private set; }

    /// <summary>
    /// Constructor for the Application object.
    /// </summary>
    public App()
    {
      // Global handler for uncaught exceptions. 
      UnhandledException += Application_UnhandledException;

      // Show graphics profiling information while debugging.
      if (System.Diagnostics.Debugger.IsAttached)
      {
        // Display the current frame rate counters.
        //Application.Current.Host.Settings.EnableFrameRateCounter = true;

        // Show the areas of the app that are being redrawn in each frame.
        //Application.Current.Host.Settings.EnableRedrawRegions = true;

        // Enable non-production analysis visualization mode, 
        // which shows areas of a page that are being GPU accelerated with a colored overlay.
        //Application.Current.Host.Settings.EnableCacheVisualization = true;
      }

      // Standard Silverlight initialization
      InitializeComponent();

      // Phone-specific initialization
      InitializePhoneApplication();

      RootFrame.Obscured += new EventHandler<ObscuredEventArgs>(RootFrame_Obscured);
      RootFrame.Unobscured += new EventHandler(RootFrame_Unobscured);
    }

    void RootFrame_Obscured(object sender, ObscuredEventArgs e)
    {
      Debug.WriteLine("Obscured");
    }

    void RootFrame_Unobscured(object sender, EventArgs e)
    {
      Debug.WriteLine("Unobscured");
    }
    
    // Code to execute when the application is launching (eg, from Start)
    // This code will not execute when the application is reactivated
    private void Application_Launching(object sender, LaunchingEventArgs e)
    {
      Debug.WriteLine("Launching");

      MainData _mainData = null;

      using (IsolatedStorageFile _isf = IsolatedStorageFile.GetUserStoreForApplication())
      {
        if (_isf.FileExists(MainData.STORAGE_FILENAME))
        {
          using (IsolatedStorageFileStream _fs = _isf.OpenFile(MainData.STORAGE_FILENAME, System.IO.FileMode.Open))
          {
            XmlSerializer _ser = new XmlSerializer(typeof(MainData));
            object _obj = _ser.Deserialize(_fs);

            if (_obj != null && _obj is MainData)
            {
              _mainData = _obj as MainData;
            }
            else
            {
              _mainData = new MainData();
            }
          }
        }
        else
        {
          _mainData = new MainData();
        }
      }

      RootFrame.DataContext = _mainData;
    }

    //private void Application_Launching(object sender, LaunchingEventArgs e)
    //{
    //  Debug.WriteLine("Launching");
    //
    //  RootFrame.DataContext = new MainData();
    //}

    // Code to execute when the application is activated (brought to foreground)
    // This code will not execute when the application is first launched
    private void Application_Activated(object sender, ActivatedEventArgs e)
    {
      Debug.WriteLine("Activated");

      MainData _mainData = null;

      if (PhoneApplicationService.Current.State.ContainsKey("MainData"))
      {
        _mainData = PhoneApplicationService.Current.State["MainData"] as MainData;

        PhoneApplicationService.Current.State.Remove("MainData");
      }

      if (_mainData != null)
      {
        RootFrame.DataContext = _mainData;
      }
      else
      {
        RootFrame.DataContext = new MainData();
      }
    }

    private void Application_Deactivated(object sender, DeactivatedEventArgs e)
    {
      Debug.WriteLine("Deactivated");

      PhoneApplicationService.Current.State.Add("MainData", RootFrame.DataContext as MainData);
    }

    // Code to execute when the application is closing (eg, user hit Back)
    // This code will not execute when the application is deactivated
    private void Application_Closing(object sender, ClosingEventArgs e)
    {
      Debug.WriteLine("Closing");
    }

    // Code to execute if a navigation fails
    private void RootFrame_NavigationFailed(object sender, NavigationFailedEventArgs e)
    {
      if (System.Diagnostics.Debugger.IsAttached)
      {
        // A navigation has failed; break into the debugger
        System.Diagnostics.Debugger.Break();
      }
    }

    // Code to execute on Unhandled Exceptions
    private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
    {
      //evita il blocco dell'applicazione
      //gestire solo se necessario
      e.Handled = true;
      
      string _error = string.Empty;
      _error = string.Format("{0} \n {1}", _error, e.ExceptionObject.Message);
      _error = string.Format("{0} \n ANID:{1}", _error, 
        Util.GetUserExt<string>("ANID"));
      _error = string.Format("{0} \n DeviceUniqueId:{1}", _error, 
        BitConverter.ToUInt32(Util.GetDeviceExt<byte[]>("DeviceUniqueId"), 0));
      _error = string.Format("{0} \n DeviceManufacturer:{1}", _error, 
        Util.GetDeviceExt<string>("DeviceManufacturer"));
      _error = string.Format("{0} \n DeviceName:{1}", _error, 
        Util.GetDeviceExt<string>("DeviceName"));
      _error = string.Format("{0} \n DeviceTotalMemory:{1}", _error, 
        Util.GetDeviceExt<long>("DeviceTotalMemory"));
      _error = string.Format("{0} \n AppCurrentMemoryUsage:{1}", _error, 
        Util.GetDeviceExt<long>("ApplicationCurrentMemoryUsage"));
      _error = string.Format("{0} \n AppPeakMemoryUsage:{1}", _error, 
        Util.GetDeviceExt<long>("ApplicationPeakMemoryUsage"));

      string _OSVersion = string.Format("{0}.{1}.{2}.{3}", 
                                        Environment.OSVersion.Version.Major,
                                        Environment.OSVersion.Version.Minor, 
                                        Environment.OSVersion.Version.Build,
                                        Environment.OSVersion.Version.Revision);
      _error = string.Format("{0} \n OSVersion:{1}", _error, _OSVersion);
      _error = string.Format("{0} \n Platform:{1}", _error, Environment.OSVersion.Platform);

      MessageBox.Show(_error);

      if (System.Diagnostics.Debugger.IsAttached)
      {
        // An unhandled exception has occurred; break into the debugger
        System.Diagnostics.Debugger.Break();
      }
    }

    #region Phone application initialization

    // Avoid double-initialization
    private bool phoneApplicationInitialized = false;

    // Do not add any additional code to this method
    private void InitializePhoneApplication()
    {
      if (phoneApplicationInitialized)
        return;

      // Create the frame but don't set it as RootVisual yet; this allows the splash
      // screen to remain active until the application is ready to render.
      RootFrame = new PhoneApplicationFrame();
      RootFrame.Navigated += CompleteInitializePhoneApplication;

      // Handle navigation failures
      RootFrame.NavigationFailed += RootFrame_NavigationFailed;

      // Ensure we don't initialize again
      phoneApplicationInitialized = true;
    }

    // Do not add any additional code to this method
    private void CompleteInitializePhoneApplication(object sender, NavigationEventArgs e)
    {
      // Set the root visual to allow the application to render
      if (RootVisual != RootFrame)
        RootVisual = RootFrame;

      // Remove this handler since it is no longer needed
      RootFrame.Navigated -= CompleteInitializePhoneApplication;
    }

    #endregion
  }
}
