using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GameStateManagement;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using XNABookGame.Object;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input.Touch;
using XNABookGame.Audio;
using Microsoft.Phone.Shell;

namespace XNABookGame.Screen
{
  class GameplayScreen : GameScreen
  {

    int score;
    Camera camera;
    Vector2 scorePosition;
    Vector3 favoriteMousePosition;
    DrawableComponent3D ground;
    DrawableComponent3D favoriteMouse;
    DrawableComponent3D[] gameplayElements;
    Cursor cursor;
    DrawableComponent3D modelUnderRay;
    int gameplayElementCount = 1;

    public GameplayScreen()
    {
      EnabledGestures = GestureType.Tap;
      scorePosition = new Vector2(10, 5);
      TransitionOnTime = TimeSpan.FromSeconds(0.0);
      TransitionOffTime = TimeSpan.FromSeconds(0.0);

      PhoneApplicationService.Current.Deactivated += new EventHandler<DeactivatedEventArgs>(Current_Deactivated);
      PhoneApplicationService.Current.Activated += new EventHandler<ActivatedEventArgs>(Current_Activated);
    }

void Current_Deactivated(object sender, DeactivatedEventArgs e)
{
  PhoneApplicationService.Current.State["score"] = score;
}
void Current_Activated(object sender, ActivatedEventArgs e)
{
  if (PhoneApplicationService.Current.State.ContainsKey("score"))
  {
    score = (int)PhoneApplicationService.Current.State["score"];
  }
}
    public override void LoadContent()
    {
      LoadAssets();
      cursor = new Cursor(this.ScreenManager.Game);
      cursor.DebugMode = true;

      ScreenManager.Game.Components.Add(cursor);
      base.LoadContent();
    }
    public override void UnloadContent()
    {
      base.UnloadContent();
      ScreenManager.Game.Components.Remove(cursor);
    }

    public void LoadAssets()
    {
      InitializeCamera();
      InitializeGround();
      InizializeGameplayElements();

    }

    private void InizializeGameplayElements()
    {
      Vector3 position = new Vector3(-65, -20, -63);
      gameplayElements = new DrawableComponent3D[12];
      int row = 0;
      for (int i = 0; i < gameplayElementCount; i++)
      {

        if (row != i / 3)
        {
          position.X = -65;
          position.Z += 35;
          row = i / 3;
        }

        position.X += 33;

        DrawableComponent3D item = new DrawableComponent3D(ScreenManager.Game, "topo")
        {
          Position = position,
          Camera = camera,
          FriendlyName = "topo_" + i
        };
        item.Initialize();

        gameplayElements[i] = item;
      }

      favoriteMouse = gameplayElements[0];
      favoriteMousePosition = favoriteMouse.Position;
    }

    float mouseMovePosition = 1;
    private void UpdateGameplayElements(GameTime gameTime)
    {
      favoriteMouse.Position = favoriteMousePosition;
      if (mouseMovePosition > 0)
      {
        mouseMovePosition += (float)(gameTime.ElapsedGameTime.TotalMilliseconds / 2900) * -1;
        favoriteMouse.Position.Y -= 60 * mouseMovePosition;
        mouseMovePosition = MathHelper.Clamp(mouseMovePosition, 0, 1);

      }
      else
      {
        mouseMovePosition = 1;
      }
      favoriteMouse.Update(gameTime);
    }

    private void DrawGameplayElements(GameTime gameTime)
    {
      for (int i = 0; i < gameplayElementCount; i++)
      {
        gameplayElements[i].Draw(gameTime);
      }
    }
    private void InitializeCamera()
    {
      // Create the camera
      camera = new Camera(ScreenManager.Game, ScreenManager.GraphicsDevice);
      camera.Initialize();
      BoundingSphereRenderer.Initialize(ScreenManager.Game.GraphicsDevice, 10);
    }


    private void InitializeGround()
    {
      ground = new DrawableComponent3D(ScreenManager.Game, "terra")
      {
        Position = Vector3.Zero,
        Camera = camera
      };

      ground.Initialize();
    }

    public override void Update(GameTime gameTime, bool otherScreenHasFocus, bool coveredByOtherScreen)
    {
      ground.Update(gameTime);
      camera.Update(gameTime);
      UpdateGameplayElements(gameTime);
    }
    /// <summary>
    /// This helper function checks to see if a ray will intersect with a model.
    /// The model's bounding spheres are used, and the model is transformed using
    /// the matrix specified in the worldTransform argument.
    /// </summary>
    /// <param name="ray">the ray to perform the intersection check with</param>
    /// <param name="model">the model to perform the intersection check with.
    /// the model's bounding spheres will be used.</param>
    /// <param name="worldTransform">a matrix that positions the model
    /// in world space</param>
    /// <param name="absoluteBoneTransforms">this array of matrices contains the
    /// absolute bone transforms for the model. this can be obtained using the
    /// Model.CopyAbsoluteBoneTransformsTo function.</param>
    /// <returns>true if the ray intersects the model.</returns>
    private static bool RayIntersectsModel(Ray ray, Model model,
        Matrix worldTransform, Matrix[] absoluteBoneTransforms)
    {
      // Each ModelMesh in a Model has a bounding sphere, so to check for an
      // intersection in the Model, we have to check every mesh.
      foreach (ModelMesh mesh in model.Meshes)
      {
        //TODO STUDY!!!!!!!!
        // the mesh's BoundingSphere is stored relative to the mesh itself.
        // (Mesh space). We want to get this BoundingSphere in terms of world
        // coordinates. To do this, we calculate a matrix that will transform
        // from coordinates from mesh space into world space....
        Matrix world = absoluteBoneTransforms[mesh.ParentBone.Index] * worldTransform;

        // ... and then transform the BoundingSphere using that matrix.
        BoundingSphere sphere = TransformBoundingSphere(mesh.BoundingSphere, worldTransform);

        // now that the we have a sphere in world coordinates, we can just use
        // the BoundingSphere class's Intersects function. Intersects returns a
        // nullable float (float?). This value is the distance at which the ray
        // intersects the BoundingSphere, or null if there is no intersection.
        // so, if the value is not null, we have a collision.
        if (sphere.Intersects(ray) != null)
        {
          return true;
        }
      }

      // if we've gotten this far, we've made it through every BoundingSphere, and
      // none of them intersected the ray. This means that there was no collision,
      // and we should return false.
      return false;
    }
    private void DrawModelNames()
    {

    }

    /// <summary>
    /// This helper function takes a BoundingSphere and a transform matrix, and
    /// returns a transformed version of that BoundingSphere.
    /// </summary>
    /// <param name="sphere">the BoundingSphere to transform</param>
    /// <param name="world">how to transform the BoundingSphere.</param>
    /// <returns>the transformed BoundingSphere/</returns>
    private static BoundingSphere TransformBoundingSphere(BoundingSphere sphere, Matrix transform)
    {
      BoundingSphere transformedSphere;

      // the transform can contain different scales on the x, y, and z components.
      // this has the effect of stretching and squishing our bounding sphere along
      // different axes. Obviously, this is no good: a bounding sphere has to be a
      // SPHERE. so, the transformed sphere's radius must be the maximum of the 
      // scaled x, y, and z radii.

      // to calculate how the transform matrix will affect the x, y, and z
      // components of the sphere, we'll create a vector3 with x y and z equal
      // to the sphere's radius...
      Vector3 scale3 = new Vector3(sphere.Radius, sphere.Radius, sphere.Radius);

      // then transform that vector using the transform matrix. we use
      // TransformNormal because we don't want to take translation into account.
      scale3 = Vector3.TransformNormal(scale3, transform);

      // scale3 contains the x, y, and z radii of a squished and stretched sphere.
      // we'll set the finished sphere's radius to the maximum of the x y and z
      // radii, creating a sphere that is large enough to contain the original 
      // squished sphere.
      transformedSphere.Radius = Math.Max(scale3.X, Math.Max(scale3.Y, scale3.Z));

      // transforming the center of the sphere is much easier. we can just use 
      // Vector3.Transform to transform the center vector. notice that we're using
      // Transform instead of TransformNormal because in this case we DO want to 
      // take translation into account.
      transformedSphere.Center = Vector3.Transform(sphere.Center, transform);

      return transformedSphere;
    }

    public override void Draw(GameTime gameTime)
    {
      ScreenManager.GraphicsDevice.Clear(Color.White);
      ScreenManager.SpriteBatch.Begin();

      // Drawing sprites changes some render states around, which don't play
      // nicely with 3d models. 
      // In particular, we need to enable the depth buffer.
      DepthStencilState depthStensilState =
          new DepthStencilState() { DepthBufferEnable = true };
      ScreenManager.GraphicsDevice.DepthStencilState = depthStensilState;

      // Draw all the game components
      ground.Draw(gameTime);
      DrawGameplayElements(gameTime);
      DrawDebugInfo(gameTime);
      DrawScore(gameTime);

      //BoundingSphere sphere = TransformBoundingSphere(mouse.Model.Meshes[0].BoundingSphere, mouse.FinalWorldTransforms);
      //BoundingSphereRenderer.Draw(sphere, camera.View, camera.Projection);

      ScreenManager.SpriteBatch.End();
      base.Draw(gameTime);

    }

    private void DrawScore(GameTime gameTime)
    {
      ScreenManager.SpriteBatch.DrawString(this.ScreenManager.Font, string.Format("Score: {0}", score), scorePosition, Color.Black);
    }

    private void DrawDebugInfo(GameTime gameTime)
    {
      if (modelUnderRay != null)
      {
        Vector3 screenSpace = ScreenManager.GraphicsDevice.Viewport.Project(
                  Vector3.Zero, camera.Projection, camera.View,
                  modelUnderRay.FinalWorldTransforms);
        Vector2 textPosition =
                    new Vector2(screenSpace.X, screenSpace.Y - 60);
        Vector2 stringCenter =
                   this.ScreenManager.Font.MeasureString(modelUnderRay.FriendlyName) / 2;
        Vector2 shadowOffset = new Vector2(1, 1);
        ScreenManager.SpriteBatch.DrawString(this.ScreenManager.Font, modelUnderRay.FriendlyName,
            textPosition + shadowOffset, Color.Black, 0.0f,
            stringCenter, 1.0f, SpriteEffects.None, 0.0f);

        // ...and then the real text on top.
        ScreenManager.SpriteBatch.DrawString(this.ScreenManager.Font, modelUnderRay.FriendlyName,
            textPosition, Color.White, 0.0f,
            stringCenter, 1.0f, SpriteEffects.None, 0.0f);
      }
    }
    public override void HandleInput(InputState input)
    {
      PlayerIndex player;

      if (input.IsNewButtonPress(Buttons.Back, ControllingPlayer, out player))
      {
        LoadingScreen.Load(ScreenManager, false, ControllingPlayer, new MainMenuScreen());
      }

      foreach (GestureSample gesture in input.Gestures)
      {
        if (gesture.GestureType == GestureType.Tap)
        {
          Ray cursorRay = cursor.CalculateCursorRay(camera.Projection, camera.View);

          for (int i = 0; i < gameplayElementCount; i++)
          {
            modelUnderRay = RayIntersectsModel(cursorRay, gameplayElements[i].Model, gameplayElements[i].FinalWorldTransforms, gameplayElements[i].AbsoluteBoneTransforms) ? gameplayElements[i] : null;

            if (modelUnderRay != null)
            {
              AudioManager.PlaySound("hit");
              score += 20;
            }
            else
            {
              score -= 10;
            }
          }
        }
      }
    }
  }
}
