﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;

namespace ASPItalia.Books.Chapter12
{
    public partial class TouchDemo : PhoneApplicationPage
    {
        public TouchDemo()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(TouchDemo_Loaded);
        }

        void TouchDemo_Loaded(object sender, RoutedEventArgs e)
        {
            Touch.FrameReported += new TouchFrameEventHandler(Touch_FrameReported);
        }

        void Touch_FrameReported(object sender, TouchFrameEventArgs e)
        {
            var points = e.GetTouchPoints(this.ContentPanel);
            var position = points[0].Position;
            foreach (var point in points)
            {
                switch (point.Action)
                {
                    case TouchAction.Down:
                    case TouchAction.Move:
                        CreateAndMoveEllipse(point);
                        break;
                    case TouchAction.Up:
                        RemoveEllipse(point);
                        break;
                }
            }
        }

        private void RemoveEllipse(TouchPoint touchPoint)
        {
            int id = touchPoint.TouchDevice.Id;
            if (this.ellipses.ContainsKey(id))
            {
                this.ContentPanel.Children.Remove(this.ellipses[id]);
                this.ellipses.Remove(id);
            }
        }

        private Dictionary<int, Ellipse> ellipses = new Dictionary<int, Ellipse>();
        private void CreateAndMoveEllipse(TouchPoint touchPoint)
        {
            int id = touchPoint.TouchDevice.Id;
            if (!this.ellipses.ContainsKey(id))
            {
                this.ellipses.Add(id, new Ellipse()
                {
                    Width = 100,
                    Height = 100,
                    Fill = this.GetRandomColor()
                });
            }

            var ellipse = this.ellipses[id];

            Canvas.SetTop(ellipse, touchPoint.Position.Y - 50);
            Canvas.SetLeft(ellipse, touchPoint.Position.X - 50);

            if (!ContentPanel.Children.Contains(ellipse))
                ContentPanel.Children.Add(ellipse);
        }

        private Random random = new Random();
        private Brush GetRandomColor()
        {
            var color = Color.FromArgb(
                255,
                (byte)random.Next(0, 255),
                (byte)random.Next(0, 255),
                (byte)random.Next(0, 255));

            return new SolidColorBrush(color);
        }
    }
}