﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Xml.Linq;
using System.IO;

namespace ASPItalia.Books.Chapter13.Server
{
    /// <summary>
    /// Rappresent a tile notification
    /// </summary>
    public class TilePushNotification : PushNotification
    {
        /// <summary>
        /// Gets or sets the background image. If the uri references a remote resource, the maximum allowed size of the tile image is 80 KB, with a maximum download time of 1 minute. If the tile image size is too large or the download time is greater than 1 minute, the default tile of your application will be displayed instead.
        /// </summary>
        /// <value>The background image.</value>
        public Uri BackgroundImage { get; set; }

        /// <summary>
        /// Gets or sets the count number to display.
        /// </summary>
        /// <value>The count.</value>
        public Int32 Count { get; set; }

        /// <summary>
        /// Gets or sets the title to display.
        /// </summary>
        /// <value>The title.</value>
        public String Title { get; set; }

        internal override void PrepareHeader(System.Net.WebHeaderCollection headers)
        {
            base.PrepareHeader(headers);

            //headers[HttpRequestHeader.ContentType] = "text/xml";
            headers["X-WindowsPhone-Target"] = "token";
            headers["X-NotificationClass"] = ((int)this.NotificationClass).ToString();
        }

        internal override byte[] GetPostData()
        {
            XDocument tileXml = new XDocument(new XDeclaration("1.0", "utf-8", "no"),
               new XElement(WPNotificationNamespace + "Notification",
                   new XElement(WPNotificationNamespace + "Tile",
                       new XElement(WPNotificationNamespace + "BackgroundImage", this.BackgroundImage),
                       new XElement(WPNotificationNamespace + "Count", this.Count),
                       new XElement(WPNotificationNamespace + "Title", this.Title))));

            return WriteXml(tileXml);
        }
    }
}
