﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Notification;
using System.IO;
using System.Text;
using System.Diagnostics;
using Microsoft.Phone.Shell;

namespace ASPItalia.Books.Chapter13
{
    public partial class MainPage : PhoneApplicationPage
    {
        private HttpNotificationChannel channel;

        public MainPage()
        {
            InitializeComponent();

            StartSchedule();

            try
            {
                StartChannel();
            }
            catch (InvalidOperationException ioe)
            {
                HandleChannelException(ioe);
            }
        }

        private void StartSchedule()
        {
            ShellTileSchedule schedule = new ShellTileSchedule();
            // EveryDay, EveryWeek, EveryMonth
            schedule.Interval = UpdateInterval.EveryHour;
            // Interval o OneTime
            schedule.Recurrence = UpdateRecurrence.Interval;
            // Ogni un'ora da adesso
            schedule.StartTime = DateTime.Now;
            schedule.RemoteImageUri = new Uri("http://miosito.it/background.png");
            // Avvio lo schedule. Stop per fermare
            schedule.Start();
        }

        private void HandleChannelException(InvalidOperationException ioe)
        {
            MessageBox.Show(ioe.Message);
        }

        private void StartChannel()
        {
            // Cerco il canale per nome, se già esiste
            channel = HttpNotificationChannel.Find("Alerts");

            if (channel == null)
            {
                // Creo il canale
                channel = new HttpNotificationChannel("Alerts", "Book");
                // Intercetto gli eventi
                AddDelegates();
                // Apro il canale
                channel.Open();
            }
            else
            {
                // Il canale esiste perciò intercetto gli eventi
                AddDelegates();
            }

            // Se l'URI è pronto lavoro sul canale
            if (channel.ChannelUri != null)
            {
                OnChannelReady();
            }
        }

        private void AddDelegates()
        {
            channel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(channel_ChannelUriUpdated);
            channel.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(channel_ErrorOccurred);
            channel.ShellToastNotificationReceived += new EventHandler<NotificationEventArgs>(channel_ShellToastNotificationReceived);
            channel.HttpNotificationReceived += new EventHandler<HttpNotificationEventArgs>(channel_HttpNotificationReceived);
        }

        private void channel_HttpNotificationReceived(object sender, HttpNotificationEventArgs e)
        {
            // Leggo il testo dallo Stream
            StreamReader sr = new StreamReader(e.Notification.Body, Encoding.UTF8);
            string text = sr.ReadToEnd();

            // Mostro a video
            this.Dispatcher.BeginInvoke(() => MessageBox.Show(text));
        }

        private void channel_ShellToastNotificationReceived(object sender, NotificationEventArgs e)
        {
            string text = e.Collection["Text1"] + " " + e.Collection["Text2"];
            this.Dispatcher.BeginInvoke(() =>
            {
                MessageBox.Show(text);
            });
        }

        private void channel_ErrorOccurred(object sender, NotificationChannelErrorEventArgs e)
        {
            switch (e.ErrorType)
            {
                case ChannelErrorType.PowerLevelChanged:
                    ChannelPowerLevel level = (ChannelPowerLevel)e.ErrorAdditionalData;

                    if (level == ChannelPowerLevel.CriticalLowPowerLevel)
                        MessageBox.Show("Livello batteria basso. Le notifiche non saranno disponibili");
                    break;
                case ChannelErrorType.ChannelOpenFailed:
                    MessageBox.Show("Impossibile aprire il canale");
                    break;
            }

            ContentPanel.Background = new SolidColorBrush(Colors.Red);
        }

        private void channel_ChannelUriUpdated(object sender, NotificationChannelUriEventArgs e)
        {
            try
            {
                this.Dispatcher.BeginInvoke(OnChannelReady);
            }
            catch (InvalidOperationException ioe)
            {
                this.Dispatcher.BeginInvoke(() => HandleChannelException(ioe));
            }
        }

        private void OnChannelReady()
        {
            // TODO: comunicazione del canale ad un nostro servizio
            // Lo stampo per essere usato poi nel test
            Debug.WriteLine(channel.ChannelUri);

            // Preparo il bind del toast
            if (!this.channel.IsShellToastBound)
            {
                if (MessageBox.Show("Abilitare le notifiche?", "Book", MessageBoxButton.OKCancel) == MessageBoxResult.OK)
                    this.channel.BindToShellToast();
            }
            // Preparo il bind del tile
            if (!this.channel.IsShellTileBound)
                this.channel.BindToShellTile();

            this.Dispatcher.BeginInvoke(() => ContentPanel.Background = new SolidColorBrush(Colors.Green));
        }
    }
}