﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Net
Imports System.Xml.Linq
Imports System.IO
Imports System.Xml

''' <summary>
''' Rappresents a notification message
''' </summary>
Public MustInherit Class PushNotification

    ''' <summary>
    ''' The optional custom header X-MessageID uniquely identifies a notification message. If it is present, the // same value is returned in the notification response
    ''' </summary>
    Public Property MessageID() As Guid
        Get
            Return m_MessageID
        End Get
        Set(ByVal value As Guid)
            m_MessageID = value
        End Set
    End Property
    Private m_MessageID As Guid

    ''' <summary>
    ''' Gets or sets the notification class.
    ''' </summary>
    ''' <value>The notification class.</value>
    Public Property NotificationClass() As NotificationClass
        Get
            Return m_NotificationClass
        End Get
        Set(ByVal value As NotificationClass)
            m_NotificationClass = value
        End Set
    End Property
    Private m_NotificationClass As NotificationClass

    Friend Shared WPNotificationNamespace As XNamespace = "WPNotification"

    ''' <summary>
    ''' Encoder used to prepare xml
    ''' </summary>
    Public Shared ReadOnly Encoding As Encoding = New UTF8Encoding(False)

    ''' <summary>
    ''' Prepares the header of the webrequest.
    ''' </summary>
    ''' <param name="headers">The headers.</param>
    Friend Overridable Sub PrepareHeader(ByVal headers As WebHeaderCollection)
        If Guid.Empty <> Me.MessageID Then
            headers("X-MessageID") = Me.MessageID.ToString()
        End If
    End Sub

    ''' <summary>
    ''' Gets the data to use for POST.
    ''' </summary>
    ''' <returns></returns>
    Friend MustOverride Function GetPostData() As Byte()

    ''' <summary>
    ''' Convert document to byte array
    ''' </summary>
    ''' <param name="doc">The doc.</param>
    ''' <returns></returns>
    Protected Function WriteXml(ByVal doc As XDocument) As Byte()
        Using stream As New MemoryStream()
            Using writer As XmlWriter = XmlWriter.Create(stream, New XmlWriterSettings() With { _
             .Encoding = Encoding _
            })
                doc.Save(writer)
                writer.Flush()

                Return stream.ToArray()
            End Using
        End Using
    End Function
End Class

''' <summary>
''' Rappresents the batching interval
''' </summary>
Public Enum NotificationClass
    Immediately = 1
    Within450Seconds = 11
    Within900Seconds = 21
End Enum