﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Notification
Imports System.IO
Imports System.Text
Imports System.Diagnostics
Imports Microsoft.Phone.Shell

Partial Public Class MainPage
    Inherits PhoneApplicationPage
    Private channel As HttpNotificationChannel

    Public Sub New()
        InitializeComponent()

        StartSchedule()

        Try
            StartChannel()
        Catch ioe As InvalidOperationException
            HandleChannelException(ioe)
        End Try
    End Sub

    Private Sub StartSchedule()
        Dim schedule As New ShellTileSchedule()
        ' EveryDay, EveryWeek, EveryMonth
        schedule.Interval = UpdateInterval.EveryHour
        ' Interval o OneTime
        schedule.Recurrence = UpdateRecurrence.Interval
        ' Ogni un'ora da adesso
        schedule.StartTime = DateTime.Now
        schedule.RemoteImageUri = New Uri("http://miosito.it/background.png")
        ' Avvio lo schedule. Stop per fermare
        schedule.Start()
    End Sub

    Private Sub HandleChannelException(ByVal ioe As InvalidOperationException)
        MessageBox.Show(ioe.Message)
    End Sub

    Private Sub StartChannel()
        ' Cerco il canale per nome, se già esiste
        channel = HttpNotificationChannel.Find("Alerts")

        If channel Is Nothing Then
            ' Creo il canale
            channel = New HttpNotificationChannel("Alerts", "Book")
            ' Intercetto gli eventi
            AddDelegates()
            ' Apro il canale
            channel.Open()
        Else
            ' Il canale esiste perciò intercetto gli eventi
            AddDelegates()
        End If

        ' Se l'URI è pronto lavoro sul canale
        If channel.ChannelUri IsNot Nothing Then
            OnChannelReady()
        End If
    End Sub

    Private Sub AddDelegates()
        AddHandler channel.ChannelUriUpdated, New EventHandler(Of NotificationChannelUriEventArgs)(AddressOf channel_ChannelUriUpdated)
        AddHandler channel.ErrorOccurred, New EventHandler(Of NotificationChannelErrorEventArgs)(AddressOf channel_ErrorOccurred)
        AddHandler channel.ShellToastNotificationReceived, New EventHandler(Of NotificationEventArgs)(AddressOf channel_ShellToastNotificationReceived)
        AddHandler channel.HttpNotificationReceived, New EventHandler(Of HttpNotificationEventArgs)(AddressOf channel_HttpNotificationReceived)
    End Sub

    Private Sub channel_HttpNotificationReceived(ByVal sender As Object, ByVal e As HttpNotificationEventArgs)
        ' Leggo il testo dallo Stream
        Dim sr As New StreamReader(e.Notification.Body, Encoding.UTF8)
        Dim text As String = sr.ReadToEnd()

        ' Mostro a video
        Me.Dispatcher.BeginInvoke(Function() MessageBox.Show(text))
    End Sub

    Private Sub channel_ShellToastNotificationReceived(ByVal sender As Object, ByVal e As NotificationEventArgs)
        Dim text As String = e.Collection("Text1") + " " + e.Collection("Text2")
        Me.Dispatcher.BeginInvoke(Sub() MessageBox.Show(text))
    End Sub

    Private Sub channel_ErrorOccurred(ByVal sender As Object, ByVal e As NotificationChannelErrorEventArgs)
        Select Case e.ErrorType
            Case ChannelErrorType.PowerLevelChanged
                Dim level As ChannelPowerLevel = DirectCast(e.ErrorAdditionalData, ChannelPowerLevel)

                If level = ChannelPowerLevel.CriticalLowPowerLevel Then
                    MessageBox.Show("Livello batteria basso. Le notifiche non saranno disponibili")
                End If
                Exit Select
            Case ChannelErrorType.ChannelOpenFailed
                MessageBox.Show("Impossibile aprire il canale")
                Exit Select
        End Select

        ContentPanel.Background = New SolidColorBrush(Colors.Red)
    End Sub

    Private Sub channel_ChannelUriUpdated(ByVal sender As Object, ByVal e As NotificationChannelUriEventArgs)
        Try
            Me.Dispatcher.BeginInvoke(AddressOf OnChannelReady)
        Catch ioe As InvalidOperationException
            Me.Dispatcher.BeginInvoke(Sub() HandleChannelException(ioe))
        End Try
    End Sub

    Private Sub OnChannelReady()
        ' TODO: comunicazione del canale ad un nostro servizio
        ' Lo stampo per essere usato poi nel test
        Debug.WriteLine(channel.ChannelUri)

        ' Preparo il bind del toast
        If Not Me.channel.IsShellToastBound Then
            If MessageBox.Show("Abilitare le notifiche?", "Book", MessageBoxButton.OKCancel) = MessageBoxResult.OK Then
                Me.channel.BindToShellToast()
            End If
        End If
        ' Preparo il bind del tile
        If Not Me.channel.IsShellTileBound Then
            Me.channel.BindToShellTile()
        End If

        Me.Dispatcher.BeginInvoke(Function() InlineAssignHelper(ContentPanel.Background, New SolidColorBrush(Colors.Green)))
    End Sub
    Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, ByVal value As T) As T
        target = value
        Return value
    End Function
End Class