﻿Imports System.IO.IsolatedStorage
Imports System.Xml.Serialization
Imports System.Diagnostics

Partial Public Class App
  Inherits Application

  ''' <summary>
  ''' Provides easy access to the root frame of the Phone Application.
  ''' </summary>
  ''' <returns>The root frame of the Phone Application.</returns>
  Public Property RootFrame As PhoneApplicationFrame

  ''' <summary>
  ''' Constructor for the Application object.
  ''' </summary>
  Public Sub New()
    ' Show graphics profiling information while debugging.
    If Diagnostics.Debugger.IsAttached Then
      ' Display the current frame rate counters.
      'Application.Current.Host.Settings.EnableFrameRateCounter = True

      ' Show the areas of the app that are being redrawn in each frame.
      'Application.Current.Host.Settings.EnableRedrawRegions = True

      ' Enable non-production analysis visualization mode, 
      ' which shows areas of a page that are being GPU accelerated with a colored overlay.
      'Application.Current.Host.Settings.EnableCacheVisualization = True
    End If

    ' Standard Silverlight initialization
    InitializeComponent()

    ' Phone-specific initialization
    InitializePhoneApplication()
  End Sub

  ' Code to execute when the application is launching (eg, from Start)
  ' This code will not execute when the application is reactivated
  Private Sub Application_Launching(ByVal sender As Object, ByVal e As LaunchingEventArgs)
    Debug.WriteLine("Launching")

    Dim _mainData As MainData = Nothing

    Using _isf As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication()
      If _isf.FileExists(MainData.STORAGE_FILENAME) Then
        Using _fs As IsolatedStorageFileStream = _isf.OpenFile(MainData.STORAGE_FILENAME, System.IO.FileMode.Open)
          Dim _ser As New XmlSerializer(GetType(MainData))
          Dim _obj As Object = _ser.Deserialize(_fs)

          If _obj IsNot Nothing AndAlso TypeOf _obj Is MainData Then
            _mainData = TryCast(_obj, MainData)
          Else
            _mainData = New MainData()
          End If
        End Using
      Else
        _mainData = New MainData()
      End If
    End Using

    RootFrame.DataContext = _mainData
  End Sub

  ' Code to execute when the application is activated (brought to foreground)
  ' This code will not execute when the application is first launched
  Private Sub Application_Activated(ByVal sender As Object, ByVal e As ActivatedEventArgs)
    Dim _mainData As MainData = Nothing

    If PhoneApplicationService.Current.State.ContainsKey("MainData") Then
      _mainData = TryCast(PhoneApplicationService.Current.State("MainData"), MainData)

      PhoneApplicationService.Current.State.Remove("MainData")
    End If

    If _mainData IsNot Nothing Then
      RootFrame.DataContext = _mainData
    Else
      RootFrame.DataContext = New MainData()
    End If
  End Sub

  ' Code to execute when the application is deactivated (sent to background)
  ' This code will not execute when the application is closing
  Private Sub Application_Deactivated(ByVal sender As Object, ByVal e As DeactivatedEventArgs)
    PhoneApplicationService.Current.State.Add("MainData", TryCast(RootFrame.DataContext, MainData))
  End Sub

  ' Code to execute when the application is closing (eg, user hit Back)
  ' This code will not execute when the application is deactivated
  Private Sub Application_Closing(ByVal sender As Object, ByVal e As ClosingEventArgs)
  End Sub

  ' Code to execute if a navigation fails
  Private Sub RootFrame_NavigationFailed(ByVal sender As Object, ByVal e As NavigationFailedEventArgs)
    If Diagnostics.Debugger.IsAttached Then
      ' A navigation has failed; break into the debugger
      Diagnostics.Debugger.Break()
    End If
  End Sub

  Public Sub Application_UnhandledException(ByVal sender As Object, ByVal e As ApplicationUnhandledExceptionEventArgs) Handles Me.UnhandledException
    Dim _error As String = String.Empty
    _error = String.Format("{0} " & vbLf & " {1}", _error, e.ExceptionObject.Message)
    _error = String.Format("{0} " & vbLf & " ANID:{1}",
                           _error,
                           Util.GetUserExt(Of String)("ANID"))
    _error = String.Format("{0} " & vbLf & " DeviceManufacturer:{1}",
                           _error,
                           Util.GetDeviceExt(Of String)("DeviceManufacturer"))
    _error = String.Format("{0} " & vbLf & " DeviceName:{1}",
                           _error,
                           Util.GetDeviceExt(Of String)("DeviceName"))
    _error = String.Format("{0} " & vbLf & " DeviceTotalMemory:{1}",
                           _error,
                           Util.GetDeviceExt(Of Long)("DeviceTotalMemory"))
    _error = String.Format("{0} " & vbLf & " AppCurrentMemoryUsage:{1}",
                           _error,
                           Util.GetDeviceExt(Of Long)("ApplicationCurrentMemoryUsage"))
    _error = String.Format("{0} " & vbLf & " AppPeakMemoryUsage:{1}",
                           _error,
                           Util.GetDeviceExt(Of Long)("ApplicationPeakMemoryUsage"))

    Dim _OSVersion As String = String.Format("{0}.{1}.{2}.{3}",
                                             Environment.OSVersion.Version.Major,
                                             Environment.OSVersion.Version.Minor,
                                             Environment.OSVersion.Version.Build,
                                             Environment.OSVersion.Version.Revision)

    _error = String.Format("{0} " & vbLf & " OSVersion:{1}", _error, _OSVersion)
    _error = String.Format("{0} " & vbLf & " Platform:{1}", _error,
                           Environment.OSVersion.Platform)

    MessageBox.Show(_error)

    e.Handled = True

    ' Show graphics profiling information while debugging.
    If Diagnostics.Debugger.IsAttached Then
      Diagnostics.Debugger.Break()
    Else
      e.Handled = True
      MessageBox.Show(e.ExceptionObject.Message & Environment.NewLine & e.ExceptionObject.StackTrace,
                      "Error", MessageBoxButton.OK)
    End If
  End Sub

#Region "Phone application initialization"
  ' Avoid double-initialization
  Private phoneApplicationInitialized As Boolean = False

  ' Do not add any additional code to this method
  Private Sub InitializePhoneApplication()
    If phoneApplicationInitialized Then
      Return
    End If

    ' Create the frame but don't set it as RootVisual yet; this allows the splash
    ' screen to remain active until the application is ready to render.
    RootFrame = New PhoneApplicationFrame()
    AddHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication

    ' Handle navigation failures
    AddHandler RootFrame.NavigationFailed, AddressOf RootFrame_NavigationFailed

    ' Ensure we don't initialize again
    phoneApplicationInitialized = True
  End Sub

  ' Do not add any additional code to this method
  Private Sub CompleteInitializePhoneApplication(ByVal sender As Object, ByVal e As NavigationEventArgs)
    ' Set the root visual to allow the application to render
    If RootVisual IsNot RootFrame Then
      RootVisual = RootFrame
    End If

    ' Remove this handler since it is no longer needed
    RemoveHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication
  End Sub
#End Region

End Class

